@@ -6,7 +6,7 @@
 */
 /* libburn wrappers for libisoburn 
 
-   Copyright 2007 - 2008  Thomas Schmitt, <scdbackup@gmx.net> 
+   Copyright 2007 - 2009  Thomas Schmitt, <scdbackup@gmx.net> 
 */
 
 /* <<< A70929 : hardcoded CD-RW with fabricated -msinfo 
@@ -24,6 +24,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <time.h>
+#include <regex.h>
 
 
 #ifndef Xorriso_standalonE
@@ -58,7 +59,7 @@ extern void *libisoburn_default_msgs_submit_handle;
 extern int libisoburn_default_msgs_submit_flag;
 
 
-int isoburn_emulate_toc(struct burn_drive *d, int flag);
+static int isoburn_emulate_toc(struct burn_drive *d, int flag);
 
 
 int isoburn_initialize(char msg[1024], int flag)
@@ -82,11 +83,11 @@ int isoburn_initialize(char msg[1024], int flag)
      iso_lib_header_version_minor
      iso_lib_header_version_micro
    If the header is too old then the following code shall cause failure of
-   cdrskin compilation rather than to allow production of a program with
+   libisoburn compilation rather than to allow production of a program with
    unpredictable bugs or memory corruption.
    The compiler messages supposed to appear in this case are:
       error: 'LIBISOFS_MISCONFIGURATION' undeclared (first use in this function)
-      error: 'INTENTIONAL_ABORT_OF_COMPILATION__HEADERFILE_libisofs_dot_h_TOO_OLD__SEE_libisoburn_dot_h_and_burn_wrap_dot_h' undeclared (first use in this function)
+      error: 'INTENTIONAL_ABORT_OF_COMPILATION__HEADERFILE_libisofs_dot_h_TOO_OLD__SEE_libisoburn_dot_h_AND_burn_wrap_dot_h' undeclared (first use in this function)
       error: 'LIBISOFS_MISCONFIGURATION_' undeclared (first use in this function)
 */
 /* The indendation is an advise of man gcc to help old compilers ignoring */
@@ -102,7 +103,7 @@ int isoburn_initialize(char msg[1024], int flag)
 
 #ifdef Isoburn_libisofs_dot_h_too_olD
 LIBISOFS_MISCONFIGURATION = 0;
-INTENTIONAL_ABORT_OF_COMPILATION__HEADERFILE_libisofs_dot_h_TOO_OLD__SEE_libisoburn_dot_h_and_burn_wrap_dot_h = 0;
+INTENTIONAL_ABORT_OF_COMPILATION__HEADERFILE_libisofs_dot_h_TOO_OLD__SEE_libisoburn_dot_h_AND_burn_wrap_dot_h = 0;
 LIBISOFS_MISCONFIGURATION_ = 0;
 #endif
 
@@ -263,6 +264,9 @@ int isoburn_is_intermediate_dvd_rw(struct burn_drive *d, int flag)
     @param flag bit0= pretent blank on overwriteable media
                 bit3= if the drive reports a -ROM profile then try to read 
                       table of content by scanning for ISO image headers.
+                bit4= do not emulate TOC on overwriteable media
+                bit5= ignore ACL from external filesystems
+                bit6= ignore POSIX Extended Attributes from external filesystems
 */ 
 static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
                                  int flag)
@@ -288,6 +292,7 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
  (*o)->msgs_submit= libisoburn_default_msgs_submit;
  (*o)->msgs_submit_handle= libisoburn_default_msgs_submit_handle;
  (*o)->msgs_submit_flag= libisoburn_default_msgs_submit_flag;
+ iso_image_set_ignore_aclea((*o)->image, (flag >> 5 ) & 3);
 
 #ifdef Hardcoded_cd_rW
  /* <<< A70929 : hardcoded CD-RW with fabricated -msinfo */
@@ -312,11 +317,13 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
        (*o)->emulation_mode= -1;
        goto ex;
      }
-     /* try to read emulated toc */
-     ret= isoburn_emulate_toc(d, 0);
-     if(ret<0) {
-       (*o)->emulation_mode= -1;
-       goto ex;
+     if(!(flag & 16)) {
+       /* try to read emulated toc */
+       ret= isoburn_emulate_toc(d, 0);
+       if(ret<0) {
+         (*o)->emulation_mode= -1;
+         goto ex;
+       }
      }
    }
 
@@ -326,9 +333,13 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
 
    if(readonly) {
      /* This might be overwriteable media in a -ROM drive */
-     ret= isoburn_emulate_toc(d, 1);
-     if(ret<0)
-       goto ex;
+     if((flag & 16)) {
+       ret= 0;
+     } else {
+       ret= isoburn_emulate_toc(d, 1);
+       if(ret<0)
+         goto ex;
+     }
      if(ret==0 && profile !=0x08 && (flag&8)) {
        /* This might also be multi-session media which do not
           get shown with a decent TOC.
@@ -373,29 +384,40 @@ ex:
                       table of content by scanning for ISO image headers.
                       (depending on media type and drive state this might 
                        help or it might make the resulting toc even worse)
+                bit4= do not emulate TOC on overwriteable media
+                bit5= ignore ACL from external filesystems
+                bit6= ignore POSIX Extended Attributes from external filesystems
 */
 int isoburn_drive_aquire(struct burn_drive_info *drive_infos[],
                          char *adr, int flag)
 {
- int ret, conv_ret, drive_grabbed= 0;
- char libburn_drive_adr[BURN_DRIVE_ADR_LEN];
+ int ret, drive_grabbed= 0;
  struct isoburn *o= NULL;
- char msg[BURN_MSGS_MESSAGE_LEN+4096];
+
+#ifndef NIX
+
+/* <<< should be obsolete by new drive addressing of libburn-0.5.2 */
+/* >>> but helps with kernel 2.4 to use /dev/sr */
+
+ int conv_ret;
+ char libburn_drive_adr[BURN_DRIVE_ADR_LEN];
 
  conv_ret= burn_drive_convert_fs_adr(adr, libburn_drive_adr);
- if(conv_ret<=0) {
-   sprintf(msg, "Unsuitable drive address: '%s'\n",adr);
-   msg[BURN_MSGS_MESSAGE_LEN-1]= 0;
-   isoburn_msgs_submit(NULL, 0x00060000, msg, 0, "FAILURE", 0);
-   ret= 0; goto ex;
- }
-
+ if(conv_ret<=0)
+   strcpy(libburn_drive_adr, adr);
  ret= burn_drive_scan_and_grab(drive_infos, libburn_drive_adr, flag&1);
+
+#else
+
+ ret= burn_drive_scan_and_grab(drive_infos, adr, flag & 1);
+
+#endif /* ! NIX */
+
  if(ret<=0)
    goto ex;
  drive_grabbed= 1;
  ret= isoburn_welcome_media(&o, (*drive_infos)[0].drive,
-                            (flag&8) | !!(flag&2));
+                            (flag & (8 | 16 | 32 | 64)) | !!(flag&2));
  if(ret<=0)
    goto ex;
 
@@ -1025,7 +1047,33 @@ int isoburn_read_iso_head(struct burn_drive *d, int lba,
  }
  ret= isoburn_read_iso_head_parse(d, buffer+32*1024, image_blocks, info,
                                   info_mode);
- return(ret);
+ if(ret<=0)
+   return(ret);
+ return(1);
+}
+
+
+int isoburn_make_toc_entry(struct isoburn *o, int *session_count, int lba,
+                           int track_blocks, int flag)
+{
+ int ret;
+ struct isoburn_toc_entry *item;
+
+ ret= isoburn_toc_entry_new(&item, o->toc, 0);
+ if(ret<=0) {
+   isoburn_msgs_submit(o, 0x00060000,
+                       "Not enough memory for emulated TOC entry object",
+                       0, "FATAL", 0);
+   return(-1);
+ }
+ if(o->toc==NULL)
+   o->toc= item;
+ (*session_count)++;
+ item->session= *session_count;
+ item->track_no= *session_count;
+ item->start_lba= lba;
+ item->track_blocks= track_blocks;
+ return(1);
 }
 
 
@@ -1037,8 +1085,8 @@ int isoburn_emulate_toc(struct burn_drive *d, int flag)
 {
  int ret, image_size= 0, lba, track_blocks, session_count= 0, read_flag= 0;
  int scan_start= 0, scan_count= 0, probe_minus_16= 0, growisofs_nwa;
+ int with_enclosure= 0;
  struct isoburn *o;
- struct isoburn_toc_entry *item;
  char msg[160], size_text[80], *sev;
  time_t start_time, last_pacifier, now;
  
@@ -1058,6 +1106,7 @@ int isoburn_emulate_toc(struct burn_drive *d, int flag)
    if(ret<=0)
      {ret= 0; goto failure;}
    lba= Libisoburn_overwriteable_starT;
+   with_enclosure= 1;
  }
  while(lba<image_size || (flag&2)) {
    now= time(NULL);
@@ -1066,7 +1115,7 @@ int isoburn_emulate_toc(struct burn_drive *d, int flag)
      if(scan_count>=10*512)
        sprintf(size_text, "%.f MB", ((double) scan_count) / 512.0);
      else
-       sprintf(size_text, "%.f MB", 2 * (double) scan_count);
+       sprintf(size_text, "%.f kB", 2 * (double) scan_count);
      sprintf(msg, "Found %d ISO sessions by scanning %s in %.f seconds",
              session_count, size_text, (double) (now - start_time));
      isoburn_msgs_submit(o, 0x00060000, msg, 0, "UPDATE", 0);
@@ -1106,27 +1155,25 @@ int isoburn_emulate_toc(struct burn_drive *d, int flag)
                "Chain of ISO session headers broken at #%d, LBA %ds",
                session_count+1, lba);
        isoburn_msgs_submit(o, 0x00060000, msg, 0, "WARNING", 0);
+
+       if(with_enclosure) {
+         ret= isoburn_make_toc_entry(o, &session_count, 0, image_size, 0);
+         if(ret<=0)
+           goto failure;
+       }
+ break; /* do not return failure */
+
      }
      {ret= 0; goto failure;}
    }
    if(ret==2) /* ISO header was found in first half block */
      lba-= 16;
-   ret= isoburn_toc_entry_new(&item, o->toc, 0);
-   if(ret<=0) {
-     isoburn_msgs_submit(o, 0x00060000,
-                         "Not enough memory for emulated TOC entry object",
-                         0, "FATAL", 0);
-     ret= -1; goto failure;
-   }
-   if(o->toc==NULL)
-     o->toc= item;
-   session_count++;
-   scan_count+= 32;
-   item->session= session_count;
-   item->track_no= session_count;
-   item->start_lba= lba;
-   item->track_blocks= track_blocks;
+
+   ret= isoburn_make_toc_entry(o, &session_count, lba, track_blocks, 0);
+   if(ret<=0)
+     goto failure;
    lba+= track_blocks;
+   scan_count+= 32;
 
    /* growisofs aligns to 16 rather than 32 */
    growisofs_nwa= lba;
@@ -1153,6 +1200,10 @@ int isoburn_emulate_toc(struct burn_drive *d, int flag)
  return(1);
 failure:;
  isoburn_toc_entry_destroy(&(o->toc), 1);
+ if(with_enclosure && o->emulation_mode == 1) {
+   session_count= 0;
+   ret= isoburn_make_toc_entry(o, &session_count, 0, image_size, 0);
+ }
  return(ret);
 }
 
@@ -1297,15 +1348,31 @@ failure:;
 int isoburn_toc_disc_get_sectors(struct isoburn_toc_disc *disc)
 {
  struct isoburn_toc_entry *t;
- int ret= 0;
+ int ret= 0, num_sessions, num_tracks;
+ struct burn_session **sessions;
+ struct burn_track **tracks;
+ struct burn_toc_entry entry;
 
  if(disc==NULL)
    return(0);
  if(disc->toc!=NULL) {
    for(t= disc->toc; t!=NULL; t= t->next)
      ret= t->start_lba + t->track_blocks;
- } else if(disc->disc!=NULL)
+ } else if(disc->disc!=NULL) {
+   sessions= burn_disc_get_sessions(disc->disc, &num_sessions);
+   if(num_sessions > 0) {
+     tracks = burn_session_get_tracks(sessions[num_sessions - 1],
+                                      &num_tracks);
+     if(num_tracks > 0) {
+       burn_track_get_entry(tracks[num_tracks - 1], &entry);
+       if(entry.extensions_valid & 1)
+         ret= entry.start_lba + entry.track_blocks;
+     }
+   }
+/*
    ret= burn_disc_get_sectors(disc->disc);
+*/
+ }
  return(ret);
 }
 
@@ -1450,12 +1517,14 @@ int isoburn_drive_set_msgs_submit(struct burn_drive *d,
 
  
 /* @param flag bit0= with adr_mode 3: adr_value might be 16 blocks too high
+               bit1= insist in seeing a disc object with at least one session
+               bit2= with adr_mode 4: use adr_value as regular expression
 */
 int isoburn_set_msc1(struct burn_drive *d, int adr_mode, char *adr_value,
                      int flag)
 {
  int ret, num_sessions, num_tracks, adr_num, i, j, total_tracks;
- int lba, best_lba, size;
+ int lba, best_lba, size, re_valid= 0;
  char volid[33], msg[160];
  struct isoburn *o;
  struct isoburn_toc_disc *disc= NULL;
@@ -1463,6 +1532,8 @@ int isoburn_set_msc1(struct burn_drive *d, int adr_mode, char *adr_value,
  struct isoburn_toc_track **tracks= NULL;
  static char mode_names[][20]= {"auto", "session", "track", "lba", "volid"};
  static int max_mode_names= 4;
+ regex_t re;
+ regmatch_t match[1];
 
  ret= isoburn_find_emulator(&o, d, 0);
  if(ret<0)
@@ -1471,7 +1542,7 @@ int isoburn_set_msc1(struct burn_drive *d, int adr_mode, char *adr_value,
    return(-1);
 
  adr_num= atoi(adr_value);
- if(adr_mode!=3) {
+ if(adr_mode!=3 || (flag & 2)) {
    disc= isoburn_toc_drive_get_disc(d);
    if(disc==NULL) {
 not_found:;
@@ -1530,6 +1601,13 @@ not_found:;
    }
  } else if(adr_mode==4) {
    /* search for volume id that is equal to adr_value */
+   if(flag & 4) {
+     ret= regcomp(&re, adr_value, 0);
+     if(ret != 0)
+       flag&= ~4;
+     else
+       re_valid= 1;
+   }
    best_lba= -1;
    for(i=0; i<num_sessions; i++) {
      tracks= isoburn_toc_session_get_tracks(sessions[i], &num_tracks);
@@ -1540,8 +1618,14 @@ not_found:;
        ret= isoburn_read_iso_head(d, lba, &size, volid, 1);
        if(ret<=0)
      continue;
-       if(strcmp(volid, adr_value)!=0)
+       if(flag & 4) {
+         ret= regexec(&re, volid, 1, match, 0);
+         if(ret != 0)
      continue;
+       } else {
+         if(strcmp(volid, adr_value)!=0)
+     continue;
+       }
        best_lba= lba;
      }
    }
@@ -1559,7 +1643,64 @@ unknown_mode:;
 ex:;
  if(disc!=NULL)
    isoburn_toc_disc_free(disc);
+ if((flag & 4) && re_valid)
+   regfree(&re);
  return(ret);
 }
 
 
+int isoburn_get_mount_params(struct burn_drive *d,
+                             int adr_mode, char *adr_value,
+                             int *lba, int *track, int *session,
+                             char volid[33], int flag)
+{
+ int msc1_mem, ret, total_tracks, num_sessions, num_tracks, i, j, track_lba;
+ int size, is_iso= 0;
+ struct isoburn *o;
+ struct isoburn_toc_disc *disc= NULL;
+ struct isoburn_toc_session **sessions= NULL;
+ struct isoburn_toc_track **tracks= NULL;
+
+ *lba= *track= *session= -1;
+ volid[0]= 0;
+ ret= isoburn_find_emulator(&o, d, 0);
+ if(ret < 0 || o == NULL)
+   return(-1);
+ msc1_mem= o->fabricated_msc1;
+ ret= isoburn_set_msc1(d, adr_mode, adr_value, 2 | (flag & 4));
+ if(ret <= 0)
+   return(ret);
+ *lba= o->fabricated_msc1;
+
+ disc= isoburn_toc_drive_get_disc(d);
+ if(disc==NULL) 
+   {ret= -1; goto ex;} /* cannot happen because checked by isoburn_set_msc1 */
+ sessions= isoburn_toc_disc_get_sessions(disc, &num_sessions);
+ if(sessions==NULL || num_sessions<=0)
+   {ret= -1; goto ex;} /* cannot happen because checked by isoburn_set_msc1 */
+ total_tracks= 0;
+ for(i=0; i<num_sessions && *session < 0; i++) {
+   tracks= isoburn_toc_session_get_tracks(sessions[i], &num_tracks);
+   if(tracks==NULL)
+ continue;
+   for(j= 0; j<num_tracks && *track < 0; j++) {
+     total_tracks++;
+     isoburn_get_track_lba(tracks[j], &track_lba, 0);
+     if(track_lba == *lba) {
+       *track= total_tracks;
+       *session= i + 1;
+     }
+   }
+ }
+ ret= isoburn_read_iso_head(d, *lba, &size, volid, 1);
+ if(ret <= 0)
+   volid[0]= 0;
+ else
+   is_iso= 1;
+
+ex:;
+ o->fabricated_msc1= msc1_mem;
+ return(2 - is_iso); 
+}
+
+
