@@ -250,6 +250,12 @@ struct isoburn_read_opts {
     unsigned int norock:1; /*< Do not read Rock Ridge extensions */
     unsigned int nojoliet:1; /*< Do not read Joliet extensions */
     unsigned int noiso1999:1; /*< Do not read ISO 9660:1999 enhanced tree */
+
+    /* ts A90121 */
+    unsigned int noaaip:1;   /* Do not read AAIP for ACL and EA */
+    unsigned int noacl:1;    /* Do not read ACL from external file objects */
+    unsigned int noea:1;     /* Do not read XFS-style EA from externals */
+
     unsigned int preferjoliet:1; 
                 /*< When both Joliet and RR extensions are present, the RR
                  *  tree is used. If you prefer using Joliet, set this to 1. */
@@ -302,6 +308,9 @@ struct isoburn_imgen_opts {
     unsigned int joliet :1;
     unsigned int iso1999 :1;
 
+    /* ts A90121 */
+    unsigned int aaip:1;   /* Write eventual AAIP info containing ACL and EA */
+
     /* relaxed constraints */
 
     /*
@@ -357,6 +366,31 @@ struct isoburn_imgen_opts {
      */
     unsigned int joliet_longer_paths :1;
 
+    /**
+     * Store timestamps as GMT rather than in local time.
+     */
+    unsigned int always_gmt :1;
+
+    /**
+     * Write Rock Ridge info as of specification RRIP-1.10 rather than
+     * RRIP-1.12: signature "RRIP_1991A" rather than "IEEE_1282",
+     *            field PX without file serial number
+     */
+    unsigned int rrip_version_1_10 :1;
+
+    /**
+     * Store as ECMA-119 Directory Record timestamp the mtime
+     * of the source rather than the image creation time.
+     */
+    unsigned int dir_rec_mtime :1;
+
+    /**
+     * Write AAIP as extension according to SUSP 1.10 rather than SUSP 1.12.
+     * I.e. without announcing it by an ER field and thus without the need
+     * to preceed the RRIP fields by an ES and to preceed the AA field by ES.
+     */
+    unsigned int aaip_susp_1_10 :1;
+
     unsigned int sort_files:1;
                 /**< If files should be sorted based on their weight. */
 
@@ -396,12 +430,19 @@ struct isoburn_imgen_opts {
     int fifo_size;
 
 
-    /** Output value: Block address of session start as evaluatedfrom media
+    /** Output value: Block address of session start as evaluated from media
                       and other options by libisoburn and libburn.
         If <0 : Invalid
         If >=0: Valid block number. Block size is always 2 KiB.
     */
     int effective_lba;
+
+    /** Output value: Block address of data section start as predicted by
+                      libisofs.
+        If < 16: Invalid
+        If >=16: Valid block number. Block size is always 2 KiB.
+    */
+    int data_start_lba;
 };
 
 
