@@ -2,7 +2,7 @@
 /*
   API definition of libisoburn.
 
-  Copyright 2007-2008 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
+  Copyright 2007-2009 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
                   and Thomas Schmitt <scdbackup@gmx.net>
 */
 
@@ -124,6 +124,22 @@ One should inquire isoburn_drive_wrote_well() to learn about overall success.
 Finally one must call isoburn_activate_session() which will complete any
 eventual multi-session emulation.
 
+
+                         Application Constraints
+
+Applications shall include libisofs/libisofs.h , libburn/libburn.h and this
+file itself: libisoburn/libisoburn.h .
+They shall link with -lisofs -lburn -lisoburn or with the .o files emerging
+from building those libraries from their sources.
+
+Applications must use 64 bit off_t, e.g. on 32-bit Linux by defining
+  #define _LARGEFILE_SOURCE
+  #define _FILE_OFFSET_BITS 64
+or take special precautions to interface with the library by 64 bit integers
+where above .h files prescribe off_t. Not to use 64 bit file i/o will keep 
+the application from producing and processing ISO images of more than 2 GB 
+size.
+
 */
 
 
@@ -200,14 +216,14 @@ void isoburn_version(int *major, int *minor, int *micro);
 */
 #define isoburn_libisofs_req_major  0
 #define isoburn_libisofs_req_minor  6
-#define isoburn_libisofs_req_micro  6
+#define isoburn_libisofs_req_micro 14 
 
 /** The minimum version of libburn to be used with this version of libisoburn
     at compile time.
     @since 0.1.0
 */
 #define isoburn_libburn_req_major  0
-#define isoburn_libburn_req_minor  5
+#define isoburn_libburn_req_minor  6
 #define isoburn_libburn_req_micro  0
 
 
@@ -243,8 +259,8 @@ int isoburn_libburn_req(int *major, int *minor, int *micro);
     @since 0.1.0
 */
 #define isoburn_header_version_major  0
-#define isoburn_header_version_minor  2
-#define isoburn_header_version_micro  2
+#define isoburn_header_version_minor  3
+#define isoburn_header_version_micro  4
 /** Note:
     Above version numbers are also recorded in configure.ac because libtool
     wants them as parameters at build time.
@@ -336,7 +352,8 @@ int isoburn_set_msgs_submit(int (*msgs_submit)(void *handle, int error_code,
                   (cdrom/burner). Thus use with driveno 0 only. On failure
                   the array has no valid elements at all.
                   The returned array should be freed via burn_drive_info_free()
-                  when the drive is no longer needed.
+                  when the drive is no longer needed. But before this is done
+                  one has to call isoburn_drive_release(drive_infos[0].drive).
     @param adr    The persistent address of the desired drive.
     @param load   1 attempt to load the disc tray. 0 no attempt,rather failure.
     @return       1 = success , 0 = drive not found , <0 = other error
@@ -353,7 +370,8 @@ int isoburn_drive_scan_and_grab(struct burn_drive_info *drive_infos[],
                   (cdrom/burner). Thus use with driveno 0 only. On failure
                   the array has no valid elements at all.
                   The returned array should be freed via burn_drive_info_free()
-                  when the drive is no longer needed.
+                  when the drive is no longer needed. But before this is done
+                  one has to call isoburn_drive_release(drive_infos[0].drive).
     @param adr    The persistent address of the desired drive.
     @param flag   bit0= attempt to load the disc tray.
                         Else: failure if not loaded.
@@ -364,6 +382,10 @@ int isoburn_drive_scan_and_grab(struct burn_drive_info *drive_infos[],
                         table of content by scanning for ISO image headers.
                         (depending on media type and drive this might
                          help or it might make the resulting toc even worse)
+                  bit4= do not emulate table of content on overwriteable media
+                  bit5= ignore ACL from external filesystems
+                  bit6= ignore POSIX Extended Attributes from external
+                        filesystems
     @return       1 = success , 0 = drive not found , <0 = other error
 */
 int isoburn_drive_aquire(struct burn_drive_info *drive_infos[],
@@ -374,6 +396,7 @@ int isoburn_drive_aquire(struct burn_drive_info *drive_infos[],
     Wrapper for: burn_drive_grab()
     @since 0.1.0
     @param drive The drive to grab. E.g. drive_infos[1].drive .
+                 Call isoburn_drive_release(drive) when it it no longer needed.
     @param load  1 attempt to load the disc tray. 0 no attempt, rather failure.
     @return      1 success, <=0 failure
 */
@@ -449,12 +472,14 @@ void isoburn_disc_erase(struct burn_drive *drive, int fast);
                     3= adr_value itself is the lba to be used
                     4= start lba of last session with volume id
                        given by adr_value
-    @parm adr_value A string describing the value to be eventually used.
+    @param adr_value A string describing the value to be eventually used.
     @param flag Bitfield for control purposes.
                 bit0= @since 0.2.2
                       with adr_mode 3: adr_value might be 16 blocks too high
                       (e.g. -C stemming from growisofs). Probe for ISO head
                       at adr_value-16 and eventually adjust setting. 
+                bit1= insist in seeing a disc object with at least one session
+                bit2= with adr_mode 4: use adr_value as regular expression
 */
 int isoburn_set_msc1(struct burn_drive *d, int adr_mode, char *adr_value,
                      int flag);
@@ -502,7 +527,8 @@ struct isoburn_toc_disc *isoburn_toc_drive_get_disc(struct burn_drive *d);
 
 
 /** Tell the number of 2048 byte blocks covered by the table of content.
-    Wrapper for: burn_disc_get_sectors()
+    This number includes the eventual gaps between sessions and tracks.
+    So this call is not really a wrapper for burn_disc_get_sectors().
     @since 0.1.6
     @param disc The master handle of the media
     @return number of blocks, <=0 indicates unknown or unreadable state
@@ -553,7 +579,7 @@ struct isoburn_toc_track **isoburn_toc_session_get_tracks(
                                       struct isoburn_toc_session *s, int *num);
 
 
-/** Obtain a copy of the entry which describes a particular itrack.
+/** Obtain a copy of the entry which describes a particular track.
     Wrapper for: burn_track_get_entry()
     @since 0.1.6
     @param s The track handle
@@ -598,6 +624,35 @@ int isoburn_read_iso_head(struct burn_drive *d, int lba,
                           int *image_blocks, char *info, int flag);
 
 
+/** Try to convert the given entity address into various entity addresses
+    which would describe it.
+    Note: Sessions and tracks are counted beginning with 1, not with 0.
+    @since 0.3.2
+    @param drive The drive where msc1 is to be set
+    @param adr_mode Determines how to interpret the input adr_value.
+                    If adr_value shall represent a number then decimal ASCII
+                    digits are expected.
+                    0= start lba of last session in TOC, ignore adr_value
+                    1= start lba of session number given by adr_value
+                    2= start lba of track given number by adr_value
+                    3= adr_value itself is the lba to be used
+                    4= start lba of last session with volume id
+                       given by adr_value
+    @param adr_value A string describing the value to be eventually used.
+    @param lba       returns the block address of the entity, -1 means invalid
+    @param track     returns the track number of the entity, -1 means invalid
+    @param session   returns the session number of the entity, -1 means invalid
+    @param volid     returns the volume id of the entity if it is a ISO session
+    @param flag      Bitfield for control purposes.
+                     bit2= with adr_mode 4: use adr_value as regular expression
+    @return          <=0 error , 1 ok, ISO session, 2 ok, not an ISO session
+*/
+int isoburn_get_mount_params(struct burn_drive *d,
+                             int adr_mode, char *adr_value,
+                             int *lba, int *track, int *session,
+                             char volid[33], int flag);
+
+
 /* ----------------------------------------------------------------------- */
 /*
 
@@ -646,6 +701,18 @@ int isoburn_ropt_destroy(struct isoburn_read_opts **o, int flag);
                      tree is used. If you prefer using Joliet, set this to 1.
                bit4= pretend_blank
                      Always create empty image.Ignore any image on input drive.
+               bit5= noaaip
+                     @since 0.3.4
+                     Do not load AAIP information from image. This information
+                     eventually contains ACL or XFS-style Extended Attributes.
+               bit6= noacl
+                     @since 0.3.4
+                     Do not obtain ACL from external filesystem objects (e.g.
+                     local filesystem files).
+               bit7= noea
+                     @since 0.3.4
+                     Do not obtain XFS-style Extended Attributes from external
+                     filesystem objects (e.g.  local filesystem files).
     @return    1 success, <=0 failure
 */
 #define isoburn_ropt_norock         1
@@ -653,6 +720,10 @@ int isoburn_ropt_destroy(struct isoburn_read_opts **o, int flag);
 #define isoburn_ropt_noiso1999      4
 #define isoburn_ropt_preferjoliet   8
 #define isoburn_ropt_pretend_blank 16
+#define isoburn_ropt_noaaip        32
+#define isoburn_ropt_noacl         64
+#define isoburn_ropt_noea         128
+
 int isoburn_ropt_set_extensions(struct isoburn_read_opts *o, int ext);
 int isoburn_ropt_get_extensions(struct isoburn_read_opts *o, int *ext);
 
@@ -728,6 +799,13 @@ int isoburn_ropt_get_input_charset(struct isoburn_read_opts *o,
 #define isoburn_ropt_has_joliet    2
 #define isoburn_ropt_has_iso1999   4
 #define isoburn_ropt_has_el_torito 8
+
+/* ts A90122 */
+/* >>> to be implemented:
+#define isoburn_ropt_has_acl          64
+#define isoburn_ropt_ihas_ea         128
+*/
+
 int isoburn_ropt_get_size_what(struct isoburn_read_opts *o,
                                uint32_t *size, int *has_what);
 
@@ -795,11 +873,16 @@ int isoburn_igopt_get_level(struct isoburn_imgen_opts *o, int *level);
                  Weaker than RockRidge, but also readable with Linux.
            bit2= iso1999
                  This is rather exotic. Better do not surprise the readers.
+           bit5= aaip
+                 The libisofs specific SUSP based extension of ECMA-119 which
+                 can encode ACL and XFS-style Extended Attributes.
     @return 1 success, <=0 failure
 */
 #define isoburn_igopt_rockridge 1
 #define isoburn_igopt_joliet    2
 #define isoburn_igopt_iso1999   4
+/* ts A90122 */
+#define isoburn_igopt_aaip     32
 int isoburn_igopt_set_extensions(struct isoburn_imgen_opts *o, int ext);
 int isoburn_igopt_get_extensions(struct isoburn_imgen_opts *o, int *ext);
 
@@ -840,6 +923,27 @@ int isoburn_igopt_get_extensions(struct isoburn_imgen_opts *o, int *ext);
            bit7= joliet_longer_paths
                  Allow paths in the Joliet tree to have more than
                  240 characters.
+           bit8= always_gmt
+                 Write timestamps as GMT although the specs prescribe local
+                 time with eventual non-zero timezone offset. Negative
+                 timezones (west of GMT) can trigger bugs in some operating
+                 systems which typically appear in mounted ISO images as if
+                 the timezone shift from GMT was applied twice
+                 (e.g. in New York 22:36 becomes 17:36).
+           bit9= rrip_version_1_10
+                 Write Rock Ridge info as of specification RRIP-1.10 rather
+                 than RRIP-1.12: signature "RRIP_1991A" rather than
+                 "IEEE_1282", field PX without file serial number.
+          bit10= dir_rec_mtime
+                 Store as ECMA-119 Directory Record timestamp the mtime
+                 of the source rather than the image creation time.
+          bit11= aaip_susp_1_10
+                 Write AAIP fields without announcing AAIP by an ER field and
+                 without distinguishing RRIP fields from the AAIP field by
+                 prefixed ES fields. This saves 5 to 10 bytes per file and
+                 might avoid problems with readers which only accept RRIP.
+                 SUSP-1.10 allows it, SUSP-1.12 frowns on it.
+
     @return 1 success, <=0 failure
 */
 #define isoburn_igopt_omit_version_numbers       1
@@ -850,6 +954,10 @@ int isoburn_igopt_get_extensions(struct isoburn_imgen_opts *o, int *ext);
 #define isoburn_igopt_allow_lowercase           32
 #define isoburn_igopt_allow_full_ascii          64
 #define isoburn_igopt_joliet_longer_paths      128
+#define isoburn_igopt_always_gmt               256
+#define isoburn_igopt_rrip_version_1_10        512
+#define isoburn_igopt_dir_rec_mtime           1024
+#define isoburn_igopt_aaip_susp_1_10          2048
 int isoburn_igopt_set_relaxed(struct isoburn_imgen_opts *o, int relax);
 int isoburn_igopt_get_relaxed(struct isoburn_imgen_opts *o, int *relax);
 
@@ -952,6 +1060,21 @@ int isoburn_igopt_get_fifo_size(struct isoburn_imgen_opts *o, int *fifo_size);
 int isoburn_igopt_get_effective_lba(struct isoburn_imgen_opts *o, int *lba);
 
 
+/** Obtain after image preparation the lowest block address of file content
+    data. Failure can occur if libisofs is too old to provide this information,
+    if the result exceeds 31 bit, or if the call is made before image
+    preparation.
+    This value cannot be set by the application but only be inquired.
+    @since 0.3.4
+    @param o          The option set to work on
+    @param lba        The block number of the session start on media.
+                      <0 means that no address has been determined yet.
+    @return 1 success, <=0 failure
+*/
+int isoburn_igopt_get_data_start(struct isoburn_imgen_opts *o, int *lba);
+
+
+
 /* ----------------------------------------------------------------------- */
 /*                      End of Options for image generation                */
 /* ----------------------------------------------------------------------- */
@@ -1051,9 +1174,10 @@ off_t isoburn_disc_available_space(struct burn_drive *d,
 
 
 /** Obtain the start block number of the most recent session on media. In
-    case of random access media this will always be 0. Succesfull return is
+    case of random access media this will normally be 0. Successfull return is
     not a guarantee that there is a ISO-9660 image at all. The call will fail,
-    nevertheless,if isoburn_disc_get_status() returns not BURN_DISC_APPENDABLE.
+    nevertheless,if isoburn_disc_get_status() returns not BURN_DISC_APPENDABLE
+    or BURN_DISC_FULL.
     Note: The result of this call may be fabricated by a previous call of
     isoburn_set_msc1() which can override the rule to load the most recent
     session.
@@ -1181,10 +1305,9 @@ int isoburn_prepare_blind_grow(struct burn_drive *d, struct burn_disc **disc,
 
 
 /**
-    Revoke isoburn_prepare_new_image() or isoburn_prepare_disc() instead of
-    running isoburn_disc_write().
+    Revoke isoburn_prepare_*() instead of running isoburn_disc_write().
     libisofs reserves resources and maybe already starts generating the
-    image stream when one of above two calls is performed. It is mandatory to
+    image stream when one of above three calls is performed. It is mandatory to
     either run isoburn_disc_write() or to revoke the preparations by the
     call described here.
     @since 0.1.0
@@ -1223,7 +1346,7 @@ void isoburn_disc_write(struct burn_write_opts *o, struct burn_disc *disc);
     Hint: If only burn_write_opts and not burn_drive is known, then the drive
           can be obtained by burn_write_opts_get_drive().
     @since 0.1.0
-    @parm d      The drive to which the track with the fifo gets burned.
+    @param d     The drive to which the track with the fifo gets burned.
     @param size  The total size of the fifo
     @param free_bytes  The current free capacity of the fifo
     @param status_text  Returns a pointer to a constant text, see below
