@@ -2,7 +2,7 @@
 /* Command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2008 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2009 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2.
 
@@ -150,6 +150,104 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag);
 
 
 
+/* The outlist stack allows to redirect the info and result messages from
+   their normal channels into a pair of string lists which can at some
+   later time be retrieved by the application.
+   These redirection caches can be stacked to allow stacked applications.
+   xorriso itself uses them for internal purposes. 
+*/
+
+/* A list item able of forming double chained lists */
+struct Xorriso_lsT;
+
+/** Maximum number of stacked redirections */
+#define Xorriso_max_outlist_stacK 32
+
+/** Enable a new redirection of info and/or result channel. The normal message
+    output and eventual older redirections will not see new messages until
+    the redirection is ended by a call to Xorriso_pull_outlists() with the
+    stack_handle value returned by this call.
+    Redirected output is not written to the files of Xorriso_option_logfile()
+    and the Xorriso_option_pkt_output() protocol will not be applied.
+    @param xorriso      The environment handle
+    @param stack_handle returns an id number which is unique as long as
+                        its redirection is stacked. It may be re-used after
+                        its redirection was pulled from the stack.
+    @param flag         Bitfield for control purposes
+                         bit0= redirect result channel
+                         bit1= redirect info channel
+                        If bit0 and bit1 are 0, both channels get redirected.
+    @return           1 on success, <=0 if failure
+*/
+int Xorriso_push_outlists(struct XorrisO *xorriso, int *stack_handle,
+                          int flag);
+
+
+/** Disable the redirection given by stack_handle. If it was the current
+    receiver of messages then switch output to the next older redirection
+    resp. to the normal channels if no redirections are stacked any more.
+    The messages collected by the disabled redirection are handed out as
+    two lists. Both lists have to be disposed via Xorriso_lst_destroy_all() 
+    when they are no longer needed.
+    The message lists are either NULL or represented by their first
+    Xorriso_lsT item.
+    @param xorriso       The environment handle
+    @param stack_handle  The id number returned by Xorriso_push_outlists()
+    @param result_list   Result and mark messages (usually directed to stdout)
+    @param info_list     Info and mark messages (usually directed to stderr)
+    @param flag          unused yet, submit 0
+    @return              1 on success, <=0 if failure
+*/
+int Xorriso_pull_outlists(struct XorrisO *xorriso, int stack_handle,
+                          struct Xorriso_lsT **result_list,
+                          struct Xorriso_lsT **info_list, int flag);
+
+
+/** Obtain the text message from the current list item.
+    @param entry    The current list item
+    @param flag     unused yet, submit 0
+    @return         Pointer to the text content of the list item.
+                    This pointer does not have to be freed.
+*/
+char *Xorriso_lst_get_text(struct Xorriso_lsT *entry, int flag);
+
+
+/** Obtain the address of the next item in the chain of messages.
+    An iteration over the output of Xorriso_pull_outlists() starts at the
+    returned result_list resp. info_list and ends when this function returns
+    NULL.
+    @param entry    The current list item
+    @param flag     unused yet, submit 0
+    @return         Pointer to the next list item or NULL if end of list.
+                    This pointer does not have to be freed.
+*/
+struct Xorriso_lsT *Xorriso_lst_get_next(struct Xorriso_lsT *entry, int flag);
+
+
+/** Obtain the address of the previous item in the chain of messages.
+    @param entry    The current list item
+    @param flag     unused yet, submit 0
+    @return         Pointer to the previous list item or NULL if start of list.
+                    This pointer does not have to be freed.
+*/
+struct Xorriso_lsT *Xorriso_lst_get_prev(struct Xorriso_lsT *entry, int flag);
+
+
+/** Destroy all list items which are directly or indirectly connected to
+    the given link item.
+    Apply this to each of the two list handles obtained by 
+    Xorriso_pull_outlists() when the lists are no longer needed.
+    @param lstring  *lstring will be freed and set to NULL.
+                    It is not dangerous to submit a pointer to a NULL-pointer.
+    @param flag     unused yet, submit 0
+    @return         -1= lstring was NULL (i.e. wrong use of this call),
+                     0= *lstring was already NULL,
+                     1= item actually disposed
+*/
+int Xorriso_lst_destroy_all(struct Xorriso_lsT **lstring, int flag);
+
+
+
 /* ---------------------------- Options API ------------------------ */
 /*    See man 1 xorriso for explanation of the particular options    */
 /*
@@ -165,6 +263,11 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag);
 /* Option -abort_on */
 int Xorriso_option_abort_on(struct XorrisO *xorriso, char *severity, int flag);
 
+
+/* Option -acl "on"|"off" */
+int Xorriso_option_acl(struct XorrisO *xorriso, char *mode, int flag);
+
+
 /* Option -add */
 /* @param flag bit0=do not report the added item
                bit1=do not reset pacifier, no final pacifier message
@@ -184,6 +287,25 @@ int Xorriso_option_alter_date(struct XorrisO *xorriso,
                               char *time_type, char *timestring,
                               int argc, char **argv, int *idx, int flag);
 
+/* Option -application_id */
+int Xorriso_option_application_id(struct XorrisO *xorriso, char *name, 
+                                  int flag);
+
+/* Option -as */
+/* @param flag bit0=do not report the added item
+               bit1=do not reset pacifier, no final pacifier message
+*/  
+int Xorriso_option_as(struct XorrisO *xorriso, int argc, char **argv,
+                      int *idx, int flag);
+
+/* Option -assert_volid */
+int Xorriso_option_assert_volid(struct XorrisO *xorriso, char *pattern,
+                                char *severity, int flag);
+
+/* Option -backslash_codes */
+int Xorriso_option_backslash_codes(struct XorrisO *xorriso, char *mode,
+                                   int flag);
+
 /* Option -ban_stdio_write */
 int Xorriso_option_ban_stdio_write(struct XorrisO *xorriso, int flag);
 
@@ -203,6 +325,20 @@ int Xorriso_option_cdi(struct XorrisO *xorriso, char *iso_rr_path, int flag);
 /* Option -cdx */
 int Xorriso_option_cdx(struct XorrisO *xorriso, char *disk_path, int flag);
 
+/* Option -charset */
+/* @param flag bit0= set in_charset
+               bit1= set out_charset
+*/
+int Xorriso_option_charset(struct XorrisO *xorriso, char *name, int flag);
+
+/* Option -check_media */
+int Xorriso_option_check_media(struct XorrisO *xorriso,
+                               int argc, char **argv, int *idx, int flag);
+
+/* Option -check_media_defaults */
+int Xorriso_option_check_media_defaults(struct XorrisO *xorriso,
+                                    int argc, char **argv, int *idx, int flag);
+
 /* Option -chgrp alias -chgrpi , chgrp_r alias chgrpi */
 /* @param flag bit0=recursive (-chgrp_r)
 */
@@ -275,9 +411,16 @@ int Xorriso_option_devices(struct XorrisO *xorriso, int flag);
 /* Option -dialog "on"|"off" */
 int Xorriso_option_dialog(struct XorrisO *xorriso, char *mode, int flag);
 
+/* Option -disk_dev_ino "on"|"off" */
+int Xorriso_option_disk_dev_ino(struct XorrisO *xorriso, char *mode, int flag);
+
 /* Option -disk_pattern "on"|"ls"|"off" */
 int Xorriso_option_disk_pattern(struct XorrisO *xorriso, char *mode, int flag);
 
+/* Option -drive_class */
+int Xorriso_option_drive_class(struct XorrisO *xorriso,
+                               char *d_class, char *pattern, int flag);
+
 /* Option -dummy "on"|"off" */
 int Xorriso_option_dummy(struct XorrisO *xorriso, char *mode, int flag);
 
@@ -308,6 +451,10 @@ int Xorriso_option_error_behavior(struct XorrisO *xorriso,
 int Xorriso_option_extract(struct XorrisO *xorriso, char *disk_path,
                        char *iso_path, int flag);
 
+/* Option -extract_cut */
+int Xorriso_option_extract_cut(struct XorrisO *xorriso, char *iso_rr_path,
+                          char *start, char *count, char *disk_path, int flag);
+
 /* Option -follow */
 int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag);
 
@@ -322,6 +469,12 @@ int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
 /* Option -fs */
 int Xorriso_option_fs(struct XorrisO *xorriso, char *size, int flag);
 
+/* Option -getfacl alias -getfacli, -getfacl_r alias -getfacl_ri */
+/* @param flag bit0=recursive -getfacl_r
+*/
+int Xorriso_option_getfacli(struct XorrisO *xorriso,
+                            int argc, char **argv, int *idx, int flag);
+
 /* Option -gid */
 int Xorriso_option_gid(struct XorrisO *xorriso, char *gid, int flag);
 
@@ -341,6 +494,10 @@ int Xorriso_option_iso_rr_pattern(struct XorrisO *xorriso, char *mode,
 /* Option -joliet "on"|"off" */
 int Xorriso_option_joliet(struct XorrisO *xorriso, char *mode, int flag);
 
+/* Option -list_delimiter */
+int Xorriso_option_list_delimiter(struct XorrisO *xorriso, char *text,
+                                  int flag);
+
 /* Option -list_formats */
 int Xorriso_option_list_formats(struct XorrisO *xorriso, int flag);
 
@@ -398,6 +555,13 @@ int Xorriso_option_mark(struct XorrisO *xorriso, char *mark, int flag);
 int Xorriso_option_mkdiri(struct XorrisO *xorriso, int argc, char **argv,
                           int *idx, int flag);
 
+/* Option -mount */
+/* @param bit0= print mount command to result channel rather than performing it
+*/
+int Xorriso_option_mount(struct XorrisO *xorriso, char *dev, char *adr_mode,
+                         char *adr, char *cmd, int flag);
+
+
 /* Option -mv alias -mvi */
 int Xorriso_option_mvi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag);
@@ -408,14 +572,15 @@ int Xorriso_option_no_rc(struct XorrisO *xorriso, int flag);
 /* Option -not_leaf */
 int Xorriso_option_not_leaf(struct XorrisO *xorriso, char *pattern, int flag);
 
-/* Option -not_list */
+/* Option -not_list , -quoted_not_list */
+/* @param flag bit0= -quoted_not_list */
 int Xorriso_option_not_list(struct XorrisO *xorriso, char *adr, int flag);
 
 /* Option -not_paths */
 int Xorriso_option_not_paths(struct XorrisO *xorriso, int argc, char **argv,
                              int *idx, int flag);
 
-/* Option -options_from_file*/
+/* Option -options_from_file */
 /* @return <=0 error , 1 = success , 3 = request to end program run */
 int Xorriso_option_options_from_file(struct XorrisO *xorriso, char *adr,
                                      int flag);
@@ -439,7 +604,8 @@ int Xorriso_option_page(struct XorrisO *xorriso, int len, int width, int flag);
 int Xorriso_option_paste_in(struct XorrisO *xorriso, char *iso_rr_path,
                           char *disk_path, char *start, char *count, int flag);
 
-/* Option -path-list */
+/* Option -path_list , -quoted_path_list */
+/* @param flag bit0= -quoted_path_list */
 int Xorriso_option_path_list(struct XorrisO *xorriso, char *adr, int flag);
 
 /* Option -pathspecs */
@@ -474,6 +640,10 @@ int Xorriso_option_pwdx(struct XorrisO *xorriso, int flag);
 /* Option -reassure "on"|"tree"|"off" */
 int Xorriso_option_reassure(struct XorrisO *xorriso, char *mode, int flag);
 
+/* Option -relax_compliance */
+int Xorriso_option_relax_compliance(struct XorrisO *xorriso, char *mode,
+                                    int flag);
+
 /* Option -report_about */
 int Xorriso_option_report_about(struct XorrisO *xorriso, char *severity, 
                                 int flag);
@@ -503,6 +673,26 @@ int Xorriso_option_rom_toc_scan(struct XorrisO *xorriso, char *mode,
 /* Option -session_log */
 int Xorriso_option_session_log(struct XorrisO *xorriso, char *path, int flag);
 
+/* Option -setfacl_list alias -setfacl_listi */
+int Xorriso_option_setfacl_listi(struct XorrisO *xorriso, char *disk_path,
+                                 int flag);
+
+/* Option -setfacl alias -setfacli , -setfacl_r  alias -setfacl_ri */
+/* @param flag bit0=recursive -setfacl_r
+*/
+int Xorriso_option_setfacli(struct XorrisO *xorriso, char *acl_text,
+                            int argc, char **argv, int *idx, int flag);
+
+/* Options -setfattr alias -setfattri, -setfattr_r alias -setfattr_ri */
+/* @param flag   bit0=recursive -setfattr_r
+*/
+int Xorriso_option_setfattri(struct XorrisO *xorriso, char *name, char *value,
+                            int argc, char **argv, int *idx, int flag);
+
+/* Option -setfattr_list alias -setfattr_listi */
+int Xorriso_option_setfattr_listi(struct XorrisO *xorriso, char *path, 
+                                  int flag);
+
 /* Option -speed */
 int Xorriso_option_speed(struct XorrisO *xorriso, char *speed, int flag);
 
@@ -551,6 +741,8 @@ int Xorriso_option_version(struct XorrisO *xorriso, int flag);
 /* Option -volid */
 int Xorriso_option_volid(struct XorrisO *xorriso, char *volid, int flag);
 
+/* Option -xattr "on"|"off" */
+int Xorriso_option_xattr(struct XorrisO *xorriso, char *mode, int flag);
 
 #endif /* Xorriso_includeD */
 
