@@ -2,7 +2,7 @@
 /* Command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2008 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2009 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2.
 
@@ -18,7 +18,7 @@
 #ifndef Xorriso_private_includeD
 #define Xorriso_private_includeD yes
 
-#define Xorriso_program_versioN "0.2.2"
+#define Xorriso_program_versioN "0.3.4"
 
 /** The source code release timestamp */
 #include "xorriso_timestamp.h"
@@ -34,23 +34,19 @@
 #define Xorriso_build_timestamP "-none-given-"
 #endif
 
-
-/* Because regex_t is mentioned in struct XorrisO */
-#ifdef Xorriso_with_regeX
 #include <regex.h>
-#endif /* Xorriso_with_regeX */
 
 
 #define Smem_malloC malloc
 #define Smem_freE free
 #define SfileadrL 4096
 
-/* <<< ??? */
-typedef int (*Cleanup_app_handler_T)();
-
 struct LinkiteM;          /* Trace of hops during symbolic link resolution */
 struct ExclusionS;        /* List of -not_* conditions */
 struct PermiteM;          /* Stack of temporarily altered access permissions */
+struct SpotlisT;          /* List of intervals with different read qualities */
+struct CheckmediajoB;     /* Parameters for Xorriso_check_media() */
+struct SectorbitmaP;      /* Distiniction between valid and invalid sectors */
 
 
 /* maximum number of history lines to be reported with -status:long_history */
@@ -65,6 +61,13 @@ struct PermiteM;          /* Stack of temporarily altered access permissions */
 #define Xorriso_rc_nuM 4
 
 
+/* Default setting for the size limit of single data files:
+   100 extents with 4 GB - 2 kB each = 400 GB - 200 kB
+*/
+#define Xorriso_default_file_size_limiT \
+        (((off_t) 400) * ((off_t) 1024*1024*1024) - (off_t) 204800)
+
+
 struct XorrisO { /* the global context of xorriso */
 
  int libs_are_started;
@@ -92,9 +95,19 @@ struct XorrisO { /* the global context of xorriso */
  int add_plainly;
  off_t split_size;
 
+ char list_delimiter[81];
+
  /* >>> put libisofs aspects here <<< */
 
  int do_joliet;
+
+ int do_aaip; /* bit0= ACL in , bit1= ACL out , bit2= EA in , bit3= EA out
+                 bit4= record dev,inode , bit5= check dev,inode
+                 bit6= omit content check if dev,inode check is conclusive
+                 bit7= omit dev check mit bit5
+              */
+
+ int relax_compliance;        /* opaque bitfield to be set by xorrisoburn */
  int do_follow_pattern;
  int do_follow_param;
  int do_follow_links;
@@ -114,8 +127,11 @@ struct XorrisO { /* the global context of xorriso */
  char volid[33];
  int volid_default;
  char loaded_volid[33];
+ char assert_volid[SfileadrL];
+ char assert_volid_sev[80];
 
  char publisher[129];
+ char application_id[129];
 
  char session_logfile[SfileadrL];
  int session_lba;
@@ -123,8 +139,14 @@ struct XorrisO { /* the global context of xorriso */
 
  /* >>> put libburn/isoburn aspects here */
 
+ struct Xorriso_lsT *drive_blacklist;
+ struct Xorriso_lsT *drive_greylist;
+ struct Xorriso_lsT *drive_whitelist;
+
  int toc_emulation_flag; /* bit0= bit3 for isoburn_drive_aquire()
                                   scan -ROM profiles for ISO sessions
+                            bit1= bit4 for isoburn_drive_aquire()
+                                  do not emulate TOC on overwriteable media
                          */
 
  int image_start_mode;  /* From what address to load the ISO image
@@ -147,12 +169,18 @@ struct XorrisO { /* the global context of xorriso */
  char indev[SfileadrL];
  void *in_drive_handle;  /* interpreted only by xorrisoburn.c */
  void *in_volset_handle; /* interpreted only by xorrisoburn.c */
+ char *in_charset;       /* The charset to interpret the filename bytes */
 
  int volset_change_pending; /* whether -commit would make sense */
  int no_volset_present;     /* set to 1 on first failure */
 
+ struct CheckmediajoB *check_media_default;
+ struct SectorbitmaP *in_sector_map; /* eventual sector validity bitmap */
+
+
  char outdev[SfileadrL];
  void *out_drive_handle; /* interpreted only by xorrisoburn.c */
+ char *out_charset;      /* The charset to produce the filename bytes for */
  int dev_fd_1; /* The fd which substitutes for /dev/fd/1 and is
                      connected to externaly perveived stdout.
                   */
@@ -168,11 +196,29 @@ struct XorrisO { /* the global context of xorriso */
  int speed;     /* in libburn units : 1000 bytes/second , 0 = Max, -1 = Min */
  int fs;        /* fifo size in 2048 byte chunks : at most 1 GB */
  int padding;   /* number of bytes to add after ISO 9660 image */
- int do_stream_recording;
+
+ int alignment; /* if > 0 : image size alignment in 2048 byt blocks */
+                /* <<< not sure whether to keep this after libisofs will have
+                       learned to pad up MBR images to full MB */
+
+ int do_stream_recording; /* 0=no, 1=yes, 2=for data, not for dir
+                             >=16 means yes with number as start LBA  */
 
  int keep_boot_image;
  int patch_isolinux_image;
+ char boot_image_bin_path[SfileadrL];
+ int boot_image_emul; /* 0=no emulation
+                        (1=emulation as hard disk)
+                        (2=emulation as floppy)
+                      */
+ char boot_image_cat_path[SfileadrL];
+ off_t boot_image_load_size;
+ int boot_image_isohybrid; /* 0=off , 1=auto , 2=on , 3=force */
 
+ /* LBA of boot image after image loading */
+ int loaded_boot_bin_lba;
+ /* Path of the catalog node after image loading */
+ char loaded_boot_cat_path[SfileadrL];
 
  /* XORRISO options */
  int allow_graft_points;
@@ -185,8 +231,20 @@ struct XorrisO { /* the global context of xorriso */
                              of self-owned directories during restore
                         */
 
- int dialog;
+ int dialog;  /* 0=off , 1=single-line , 2=multi-line */
 
+ int bsl_interpretation;
+                     /* whether to run input through Sfile_bsl_interpreter():
+                        bit0-1= dialog and quoted file reading
+                          0= no interpretation, leave unchanged
+                          1= only inside double quotes
+                          2= outside single quotes
+                          3= everywhere
+                        bit2-3= reserved as future expansion of bit0-1
+                        bit4= interpretation within program start arguments
+                        bit5= perform backslash encoding with results
+                        bit6= perform backslash encoding with info texts
+                     */
 
  /* Pattern matching facility. It still carries legacy from scdbackup/askme.c
     but is fully functional for xorriso.
@@ -212,6 +270,8 @@ struct XorrisO { /* the global context of xorriso */
 
  int temp_mem_limit;
 
+ off_t file_size_limit;
+
  struct ExclusionS *disk_exclusions;
  int disk_excl_mode; /* bit0= on (else off)
                         bit1= parameter too (else rekursion only)
@@ -226,7 +286,17 @@ struct XorrisO { /* the global context of xorriso */
  char mark_text[SfileadrL]; /* ( stdout+stderr, M: ) */
  int packet_output;
  char logfile[4][SfileadrL];
+ FILE *logfile_fp[4];
+ FILE *pktlog_fp;
+ struct Xorriso_lsT *result_msglists[Xorriso_max_outlist_stacK];
+ struct Xorriso_lsT *info_msglists[Xorriso_max_outlist_stacK];
+ int msglist_flags[Xorriso_max_outlist_stacK]; /* bit0= result is redirected
+                                                  bit1= info is redirected
+                                               */
+ int msglist_stackfill;
+
  int status_history_max; /* for -status long_history */
+ 
 
  char report_about_text[20];
  int report_about_severity;
@@ -241,6 +311,7 @@ struct XorrisO { /* the global context of xorriso */
  FILE *errfile_fp;
 
  int img_read_error_mode; /* 0=best_effort , 1=failure , 2=fatal */
+ int extract_error_mode; /* 0=best_effort , 1=keep , 2=delete */
 
  char return_with_text[20];
  int return_with_severity;
@@ -249,10 +320,8 @@ struct XorrisO { /* the global context of xorriso */
  char eternal_problem_status_text[20];
 
  /* temporary search facilities */
-#ifdef Xorriso_with_regeX
  regex_t *re;
  regmatch_t match[1]; 
-#endif /* Xorriso_with_regeX */
  char **re_constants;
  int re_count;
  int re_fill;
@@ -289,7 +358,7 @@ struct XorrisO { /* the global context of xorriso */
  struct PermiteM *perm_stack; /* Temporarily altered dir access permissions */
 
  /* result (stdout, R: ) */
- char result_line[5*SfileadrL];
+ char result_line[10*SfileadrL];
  int result_line_counter;
  int result_page_counter;
  int result_open_line_len;
@@ -322,7 +391,7 @@ int Xorriso_regexec(struct XorrisO *xorriso, char *to_match, int *failed_at,
 int Xorriso_prepare_expansion_pattern(struct XorrisO *xorriso, char *pattern,
                                       int flag);
 
-int Xorriso__mode_to_perms(mode_t st_mode, char perms[10], int flag);
+int Xorriso__mode_to_perms(mode_t st_mode, char perms[11], int flag);
 
 int Xorriso_much_too_long(struct XorrisO *xorriso, int len, int flag);
 
@@ -455,6 +524,57 @@ int Xorriso_auto_chmod(struct XorrisO *xorriso, char *disk_path, int flag);
 int Xorriso_protect_stdout(struct XorrisO *xorriso, int flag);
 
 
+/* @param flag bit0= mark untested areas as valid
+*/
+int Xorriso_spotlist_to_sectormap(struct XorrisO *xorriso,
+                                  struct SpotlisT *spotlist,
+                                  int read_chunk,
+                                  struct SectorbitmaP **map,
+                                  int flag);
+
+int Xorriso_toc_to_string(struct XorrisO *xorriso, char **toc_text, int flag);
+
+
+/* @param flag bit0+1= what to aquire after giving up outdev
+                       0=none, 1=indev, 2=outdev, 3=both
+*/
+int Xorriso_reaquire_outdev(struct XorrisO *xorriso, int flag);
+
+
+/* Opens the -check_media data copy in for reading and writing
+*/
+int Xorriso_open_job_data_to(struct XorrisO *xorriso,
+                             struct CheckmediajoB *job, int flag);
+
+int Xorriso_no_findjob(struct XorrisO *xorriso, char *cmd, int flag);
+
+
+int Xorriso_make_mount_cmd(struct XorrisO *xorriso, char *cmd,
+                           int lba, int track, int session, char *volid,
+                           char *devadr, char result[SfileadrL], int flag);
+
+
+/* @param flag bit0= use env_path to find the desired program
+*/
+int Xorriso_execv(struct XorrisO *xorriso, char *cmd, char *env_path,
+                  int *status, int flag);
+
+int Xorriso_is_in_patternlist(struct XorrisO *xorriso,
+                        struct Xorriso_lsT *patternlist, char *path, int flag);
+
+char *Xorriso_get_pattern(struct XorrisO *xorriso,
+                         struct Xorriso_lsT *patternlist, int index, int flag);
+
+
+/* Normalize ACL and sort apart "access" ACL from "default" ACL.
+ */
+int Xorriso_normalize_acl_text(struct XorrisO *xorriso, char *in_text,
+                    char **access_acl_text, char **default_acl_text, int flag);
+
+int Xorriso_path_setfattr(struct XorrisO *xorriso, void *in_node, char *path,
+                       char *name, size_t value_length, char *value, int flag);
+
+
 int Sfile_str(char target[SfileadrL], char *source, int flag);
 
 double Sfile_microtime(int flag);
@@ -490,6 +610,65 @@ int Sfile_count_components(char *path, int flag);
 */
 int Sfile_type(char *filename, int flag);
 
+/* @param flag bit0= only encode inside quotes
+               bit1= encode < 32 outside quotes except 7, 8, 9, 10, 12, 13
+               bit2= encode in any case above 126
+               bit3= encode in any case shellsafe:
+                     <=42 , 59, 60, 62, 63, 92, 94, 96, >=123
+*/
+int Sfile_bsl_encoder(char **result, char *text, size_t text_len, int flag);
+
+
+struct Xorriso_lsT {
+  char *text;
+  struct Xorriso_lsT *prev,*next;
+};
+
+/** Create a new list item with arbitrary byte content.
+    @param lstring  The newly created object or NULL on failure
+    @param data     An array of bytes to be copied into the new object
+    @param data_len Number of bytes to be copied
+    @param link     Xorriso_lsT object to which the new object shall be linked
+    @param flag Bitfield for control purposes
+                bit0= insert before link rather than after it
+                bit1= do not copy data (e.g. because *data is invalid)
+    @return <=0 error, 1 ok
+*/
+int Xorriso_lst_new_binary(struct Xorriso_lsT **lstring, char *data,
+                           int data_len, struct Xorriso_lsT *link, int flag);
+
+
+/** Create a new list item with a 0-terminated text as content.
+    @param lstring  The newly created object or NULL on failure
+    @param text     A 0-terminated array of bytes
+    @param link     Xorriso_lsT object to which the new object shall be linked
+    @param flag Bitfield for control purposes
+                bit0= insert before link rather than after it
+    @return <=0 error, 1 ok
+*/
+int Xorriso_lst_new(struct Xorriso_lsT **lstring, char *text,
+                    struct Xorriso_lsT *link, int flag);
+
+
+/** Create a new list item at the end of a given list.
+    @param lstring  Contains as input a pointer to a pointer to any existing
+                    list item. As output this list item pointer will be
+                    changed to the address of the new list item.
+    @param data     An array of bytes to be copied into the new object
+    @param data_len Number of bytes to be copied
+    @param flag     unused yet, submit 0
+    @return <=0 error, 1 ok
+*/
+int Xorriso_lst_append_binary(struct Xorriso_lsT **entry,
+                              char *data, int data_len, int flag);
+
+
+/** Destroy a single list item and connect its eventual list neighbors.
+    @param lstring  pointer to the pointer to be freed and set to NULL
+    @param flag     unused yet, submit 0
+    @return 0= *lstring was alredy NULL, 1= ok
+*/
+int Xorriso_lst_destroy(struct Xorriso_lsT **lstring, int flag);
 
 
 char *Text_shellsafe(char *in_text, char *out_text, int flag);
@@ -504,6 +683,9 @@ char *Ftypetxt(mode_t st_mode, int flag);
 */
 char *Ftimetxt(time_t t, char timetext[40], int flag);
 
+int System_uname(char **sysname, char **release, char **version, 
+                 char **machine, int flag);
+
 
 struct DirseQ;
 
@@ -519,6 +701,12 @@ int Linkitem_reset_stack(struct LinkiteM **o, struct LinkiteM *to, int flag);
 
 struct FindjoB;
 
+
+int Findjob_new(struct FindjoB **o, char *start_path, int flag);
+
+int Findjob_destroy(struct FindjoB **job, int flag);
+
+
 /* @return 0=no match , 1=match , <0 = error
 */
 int Findjob_test(struct FindjoB *job, char *name, 
@@ -531,7 +719,7 @@ int Findjob_get_action(struct FindjoB *o, int flag);
 
 /* @return <0 error, >=0 see xorriso.c struct FindjoB.action
 */ 
-int Findjob_get_action_parms(struct FindjoB *o, char **target,
+int Findjob_get_action_parms(struct FindjoB *o, char **target, char **text_2,
                              uid_t *user, gid_t *group,
                              mode_t *mode_and, mode_t *mode_or,
                              int *type, time_t *date, struct FindjoB **subjob,
@@ -557,8 +745,32 @@ int Findjob_set_action_ad(struct FindjoB *o, int type, time_t date, int flag);
 
 int Findjob_set_start_path(struct FindjoB *o, char *start_path, int flag);
 
+int Findjob_set_action_found_path(struct FindjoB *o, int flag);
+
 int Findjob_get_start_path(struct FindjoB *o, char **start_path, int flag);
 
+int Findjob_set_lba_range(struct FindjoB *o, int start_lba, int count,
+                          int flag);
+
+int Findjob_get_lba_damage_filter(struct FindjoB *o, int *start_lba,
+                                  int *end_lba, int *damage_filter, int flag);
+
+int Findjob_get_commit_filter(struct FindjoB *o, int *commit_filter, int flag);
+
+int Findjob_get_acl_filter(struct FindjoB *o, int *acl_filter, int flag);
+
+int Findjob_get_xattr_filter(struct FindjoB *o, int *xattr_filter, int flag);
+
+int Findjob_get_aaip_filter(struct FindjoB *o, int *aaip_filter, int flag);
+
+int Findjob_set_wanted_node(struct FindjoB *o, void *wanted_node, int flag);
+
+int Findjob_get_wanted_node(struct FindjoB *o, void **wanted_node, int flag);
+
+int Findjob_set_found_path(struct FindjoB *o, char *path, int flag);
+
+int Findjob_get_found_path(struct FindjoB *o, char **path, int flag);
+
 
 struct SplitparT;
 
@@ -582,33 +794,6 @@ int Splitpart__compose(char *adr, int partno, int total_parts,
 
 int Splitparts_sort(struct SplitparT *o, int count, int flag);
 
-struct LstrinG {
-  char *text;
-  struct LstrinG *prev,*next;
-};
- 
-int Lstring_destroy(struct LstrinG **lstring, int flag);
-
-int Lstring_destroy_all(struct LstrinG **lstring, int flag);
-
-/*
- @param flag Bitfield for control purposes
-        bit0= insert before link rather than after it 
-        bit1= do not copy data (e.g. because *data is invalid)
-*/
-int Lstring_new_binary(struct LstrinG **lstring, char *data, int data_len,
-                       struct LstrinG *link, int flag);
-
-/* 
- @param flag Bitfield for control purposes
-             bit0= insert before link rather than after it
-*/
-int Lstring_new(struct LstrinG **lstring, char *text, struct LstrinG *link,
-                int flag);
-
-int Lstring_append_binary(struct LstrinG **entry, char *data, int data_len,
-                           int flag);
-
 
 int Permstack_push(struct PermiteM **o, char *disk_path, struct stat *stbuf,
                    int flag);
@@ -617,5 +802,69 @@ int Permstack_pop(struct PermiteM **o, struct PermiteM *stopper,
                   struct XorrisO *xorriso, int flag);
 
 
+int Spotlist_new(struct SpotlisT **o, int flag);
+
+int Spotlist_destroy(struct SpotlisT **o, int flag);
+
+int Spotlist_add_item(struct SpotlisT *o, int start_lba, int blocks,
+                      int quality, int flag);
+
+int Spotlist_count(struct SpotlisT *o, int flag);
+
+int Spotlist_block_count(struct SpotlisT *o, int flag);
+
+int Spotlist_sector_size(struct SpotlisT *o, int read_chunk, int flag);
+
+int Spotlist_get_item(struct SpotlisT *o, int idx,
+                      int *start_lba, int *blocks, int *quality, int flag);
+
+char *Spotlist__quality_name(int quality, char name[80], int flag);
+
+
+#define Xorriso_read_quality_gooD         0x7fffffff
+#define Xorriso_read_quality_sloW         0x60000000
+#define Xorriso_read_quality_partiaL      0x50000000
+#define Xorriso_read_quality_valiD        0x40000000
+#define Xorriso_read_quality_untesteD     0x3fffffff
+#define Xorriso_read_quality_invaliD      0x3ffffffe
+#define Xorriso_read_quality_tao_enD      0x28000000
+#define Xorriso_read_quality_off_tracK    0x20000000
+#define Xorriso_read_quality_unreadablE   0x00000000
+
+
+int Checkmediajob_new(struct CheckmediajoB **o, int flag);
+
+int Checkmediajob_destroy(struct CheckmediajoB **o, int flag);
+
+int Checkmediajob_copy(struct CheckmediajoB *from, struct CheckmediajoB *to,
+                       int flag);
+
+int Sectorbitmap_new(struct SectorbitmaP **o, int sectors, int sector_size,
+                     int flag);
+int Sectorbitmap_destroy(struct SectorbitmaP **o, int flag);
+int Sectorbitmap_from_file(struct SectorbitmaP **o, char *path, char *msg,
+                           int *os_errno, int flag);
+int Sectorbitmap_to_file(struct SectorbitmaP *o, char *path, char *info,
+                         char *msg, int *os_errno, int flag);
+int Sectorbitmap_set(struct SectorbitmaP *o, int sector, int flag);
+int Sectorbitmap_set_range(struct SectorbitmaP *o,
+                           int start_sector, int sectors, int flag);
+int Sectorbitmap_is_set(struct SectorbitmaP *o, int sector, int flag);
+int Sectorbitmap_bytes_are_set(struct SectorbitmaP *o,
+                               off_t start_byte, off_t end_byte, int flag);
+
+int Sectorbitmap_get_layout(struct SectorbitmaP *o,
+                           int *sectors, int *sector_size, int flag);
+
+int Sectorbitmap_copy(struct SectorbitmaP *from, struct SectorbitmaP *to,
+                      int flag);
+
+/* bit0= append (text!=NULL) */
+int Sregex_string(char **handle, char *text, int flag);
+
+/* @param flag bit0= only test expression whether compilable
+*/
+int Sregex_match(char *pattern, char *text, int flag);
+
 #endif /* Xorriso_private_includeD */
 
